@@ -36,8 +36,6 @@
 #include "tool_libinfo.h"
 #include "tool_strdup.h"
 
-static char *parse_filename(const char *ptr, size_t len);
-
 #ifdef _WIN32
 #define BOLD    "\x1b[1m"
 #define BOLDOFF "\x1b[22m"
@@ -53,340 +51,6 @@ static char *parse_filename(const char *ptr, size_t len);
 #define LINKOFF LINK LINKST
 #endif
 
-#ifdef LINK
-static void write_linked_location(CURL *curl, const char *location,
-                                  size_t loclen, FILE *stream);
-#endif
-
-int tool_write_headers(struct HdrCbData *hdrcbdata, FILE *stream)
-{
-  struct curl_slist *h = hdrcbdata->headlist;
-  int rc = 1;
-  while(h) {
-    /* not "handled", just show it */
-    size_t len = strlen(h->data);
-    if(len != fwrite(h->data, 1, len, stream))
-      goto fail;
-    h = h->next;
-  }
-  rc = 0; /* success */
-fail:
-  curl_slist_free_all(hdrcbdata->headlist);
-  hdrcbdata->headlist = NULL;
-  return rc;
-}
-
-/*
-** callback for CURLOPT_HEADERFUNCTION
-*
-* 'size' is always 1
-*/
-size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
-{
-  struct per_transfer *per = userdata;
-  struct HdrCbData *hdrcbdata = &per->hdrcbdata;
-  struct OutStruct *outs = &per->outs;
-  struct OutStruct *heads = &per->heads;
-  struct OutStruct *etag_save = &per->etag_save;
-  const char *str = ptr;
-  const size_t cb = size * nmemb;
-  const char *end = (char *)ptr + cb;
-  const char *scheme = NULL;
-
-  if(!per->config)
-    return CURL_WRITEFUNC_ERROR;
-
-#ifdef DEBUGBUILD
-  if(size * nmemb > (size_t)CURL_MAX_HTTP_HEADER) {
-    warnf("Header data exceeds write limit");
-    return CURL_WRITEFUNC_ERROR;
-  }
-#endif
-
-#ifdef _WIN32
-  /* Discard incomplete UTF-8 sequence buffered from body */
-  if(outs->utf8seq[0])
-    memset(outs->utf8seq, 0, sizeof(outs->utf8seq));
-#endif
-
-  /*
-   * Write header data when curl option --dump-header (-D) is given.
-   */
-
-  if(per->config->headerfile && heads->stream) {
-    size_t rc = fwrite(ptr, size, nmemb, heads->stream);
-    if(rc != nmemb)
-      return rc;
-    /* flush the stream to send off what we got earlier */
-    if(fflush(heads->stream)) {
-      errorf("Failed writing headers to %s", per->config->headerfile);
-      return CURL_WRITEFUNC_ERROR;
-    }
-  }
-
-  curl_easy_getinfo(per->curl, CURLINFO_SCHEME, &scheme);
-  scheme = proto_token(scheme);
-  if((scheme == proto_http || scheme == proto_https)) {
-    long response = 0;
-    curl_easy_getinfo(per->curl, CURLINFO_RESPONSE_CODE, &response);
-
-    if((response / 100 != 2) && (response / 100 != 3))
-      /* only care about etag and content-disposition headers in 2xx and 3xx
-         responses */
-      ;
-    /*
-     * Write etag to file when --etag-save option is given.
-     */
-    else if(per->config->etag_save_file && etag_save->stream &&
-            /* match only header that start with etag (case insensitive) */
-            checkprefix("etag:", str)) {
-      const char *etag_h = &str[5];
-      const char *eot = end - 1;
-      if(*eot == '\n') {
-        while(ISBLANK(*etag_h) && (etag_h < eot))
-          etag_h++;
-        while(ISSPACE(*eot))
-          eot--;
-
-        if(eot >= etag_h) {
-          size_t etag_length = eot - etag_h + 1;
-          /*
-           * Truncate the etag save stream, it can have an existing etag value.
-           */
-#ifdef HAVE_FTRUNCATE
-          if(ftruncate(fileno(etag_save->stream), 0)) {
-            return CURL_WRITEFUNC_ERROR;
-          }
-#else
-          if(fseek(etag_save->stream, 0, SEEK_SET)) {
-            return CURL_WRITEFUNC_ERROR;
-          }
-#endif
-
-          fwrite(etag_h, 1, etag_length, etag_save->stream);
-          /* terminate with newline */
-          fputc('\n', etag_save->stream);
-          (void)fflush(etag_save->stream);
-        }
-      }
-    }
-
-    /*
-     * This callback sets the filename where output shall be written when
-     * curl options --remote-name (-O) and --remote-header-name (-J) have
-     * been simultaneously given and additionally server returns an HTTP
-     * Content-Disposition header specifying a filename property.
-     */
-
-    else if(hdrcbdata->honor_cd_filename) {
-      if((cb > 20) && checkprefix("Content-disposition:", str)) {
-        const char *p = str + 20;
-
-        /* look for the 'filename=' parameter
-           (encoded filenames (*=) are not supported) */
-        for(;;) {
-          char *filename;
-          size_t len;
-
-          while((p < end) && *p && !ISALPHA(*p))
-            p++;
-          if(p > end - 9)
-            break;
-
-          if(memcmp(p, "filename=", 9)) {
-            /* no match, find next parameter */
-            while((p < end) && *p && (*p != ';'))
-              p++;
-            if((p < end) && *p)
-              continue;
-            else
-              break;
-          }
-          p += 9;
-
-          len = cb - (size_t)(p - str);
-          filename = parse_filename(p, len);
-          if(filename) {
-            if(outs->stream) {
-              /* indication of problem, get out! */
-              curlx_free(filename);
-              return CURL_WRITEFUNC_ERROR;
-            }
-
-            if(per->config->output_dir) {
-              outs->filename = curl_maprintf("%s/%s", per->config->output_dir,
-                                             filename);
-              curlx_free(filename);
-              if(!outs->filename)
-                return CURL_WRITEFUNC_ERROR;
-            }
-            else
-              outs->filename = filename;
-
-            outs->is_cd_filename = TRUE;
-            outs->regular_file = TRUE;
-            outs->fopened = FALSE;
-            outs->alloc_filename = TRUE;
-            hdrcbdata->honor_cd_filename = FALSE; /* done now! */
-            if(!tool_create_output_file(outs, per->config))
-              return CURL_WRITEFUNC_ERROR;
-            if(tool_write_headers(&per->hdrcbdata, outs->stream))
-              return CURL_WRITEFUNC_ERROR;
-          }
-          break;
-        }
-        if(!outs->stream && !tool_create_output_file(outs, per->config))
-          return CURL_WRITEFUNC_ERROR;
-        if(tool_write_headers(&per->hdrcbdata, outs->stream))
-          return CURL_WRITEFUNC_ERROR;
-      } /* content-disposition handling */
-
-      if(hdrcbdata->honor_cd_filename &&
-         hdrcbdata->config->show_headers) {
-        /* still awaiting the Content-Disposition header, store the header in
-           memory. Since it is not null-terminated, we need an extra dance. */
-        char *clone = curl_maprintf("%.*s", (int)cb, str);
-        if(clone) {
-          struct curl_slist *old = hdrcbdata->headlist;
-          hdrcbdata->headlist = curl_slist_append(old, clone);
-          curlx_free(clone);
-          if(!hdrcbdata->headlist) {
-            curl_slist_free_all(old);
-            return CURL_WRITEFUNC_ERROR;
-          }
-        }
-        else {
-          curl_slist_free_all(hdrcbdata->headlist);
-          hdrcbdata->headlist = NULL;
-          return CURL_WRITEFUNC_ERROR;
-        }
-        return cb; /* done for now */
-      }
-    }
-  }
-  if(hdrcbdata->config->writeout) {
-    char *value = memchr(ptr, ':', cb);
-    if(value) {
-      if(per->was_last_header_empty)
-        per->num_headers = 0;
-      per->was_last_header_empty = FALSE;
-      per->num_headers++;
-    }
-    else if(ptr[0] == '\r' || ptr[0] == '\n')
-      per->was_last_header_empty = TRUE;
-  }
-  if(hdrcbdata->config->show_headers && !outs->out_null &&
-     (scheme == proto_http || scheme == proto_https ||
-      scheme == proto_rtsp || scheme == proto_file)) {
-    /* bold headers only for selected protocols */
-    char *value = NULL;
-
-    if(!outs->stream && !tool_create_output_file(outs, per->config))
-      return CURL_WRITEFUNC_ERROR;
-
-    if(global->isatty &&
-#ifdef _WIN32
-       tool_term_has_bold &&
-#endif
-       global->styled_output)
-      value = memchr(ptr, ':', cb);
-    if(value) {
-      size_t namelen = value - ptr;
-      curl_mfprintf(outs->stream, BOLD "%.*s" BOLDOFF ":", (int)namelen, ptr);
-#ifndef LINK
-      fwrite(&value[1], cb - namelen - 1, 1, outs->stream);
-#else
-      if(curl_strnequal("Location", ptr, namelen)) {
-        write_linked_location(per->curl, &value[1], cb - namelen - 1,
-                              outs->stream);
-      }
-      else
-        fwrite(&value[1], cb - namelen - 1, 1, outs->stream);
-#endif
-    }
-    else
-      /* not "handled", just show it */
-      fwrite(ptr, cb, 1, outs->stream);
-  }
-  return cb;
-}
-
-/*
- * Copies a filename part and returns an ALLOCATED data buffer.
- */
-static char *parse_filename(const char *ptr, size_t len)
-{
-  char *copy;
-  char *p;
-  char *q;
-  char stop = '\0';
-
-  copy = memdup0(ptr, len);
-  if(!copy)
-    return NULL;
-
-  p = copy;
-  if(*p == '\'' || *p == '"') {
-    /* store the starting quote */
-    stop = *p;
-    p++;
-  }
-  else
-    stop = ';';
-
-  /* scan for the end letter and stop there */
-  q = strchr(p, stop);
-  if(q)
-    *q = '\0';
-
-  /* if the filename contains a path, only use filename portion */
-  q = strrchr(p, '/');
-  if(q) {
-    p = q + 1;
-    if(!*p) {
-      tool_safefree(copy);
-      return NULL;
-    }
-  }
-
-  /* If the filename contains a backslash, only use filename portion. The idea
-     is that even systems that do not handle backslashes as path separators
-     probably want the path removed for convenience. */
-  q = strrchr(p, '\\');
-  if(q) {
-    p = q + 1;
-    if(!*p) {
-      tool_safefree(copy);
-      return NULL;
-    }
-  }
-
-  /* make sure the filename does not end in \r or \n */
-  q = strchr(p, '\r');
-  if(q)
-    *q = '\0';
-
-  q = strchr(p, '\n');
-  if(q)
-    *q = '\0';
-
-  if(copy != p)
-    memmove(copy, p, strlen(p) + 1);
-
-#if defined(_WIN32) || defined(MSDOS)
-  {
-    char *sanitized;
-    SANITIZEcode sc = sanitize_file_name(&sanitized, copy, 0);
-    tool_safefree(copy);
-    if(sc)
-      return NULL;
-    copy = sanitized;
-  }
-#endif /* _WIN32 || MSDOS */
-
-  return copy;
-}
-
 #ifdef LINK
 /*
  * Treat the Location: header specially, by writing a special escape
@@ -480,3 +144,356 @@ locdone:
   }
 }
 #endif
+
+/*
+ * Copies a filename part and returns an ALLOCATED data buffer.
+ */
+static char *parse_filename(const char *ptr, size_t len)
+{
+  char *copy;
+  char *p;
+  char *q;
+  char stop = '\0';
+
+  copy = memdup0(ptr, len);
+  if(!copy)
+    return NULL;
+
+  p = copy;
+  if(*p == '\'' || *p == '"') {
+    /* store the starting quote */
+    stop = *p;
+    p++;
+  }
+  else
+    stop = ';';
+
+  /* scan for the end letter and stop there */
+  q = strchr(p, stop);
+  if(q)
+    *q = '\0';
+
+  /* if the filename contains a path, only use filename portion */
+  q = strrchr(p, '/');
+  if(q) {
+    p = q + 1;
+    if(!*p) {
+      tool_safefree(copy);
+      return NULL;
+    }
+  }
+
+  /* If the filename contains a backslash, only use filename portion. The idea
+     is that even systems that do not handle backslashes as path separators
+     probably want the path removed for convenience. */
+  q = strrchr(p, '\\');
+  if(q) {
+    p = q + 1;
+    if(!*p) {
+      tool_safefree(copy);
+      return NULL;
+    }
+  }
+
+  /* make sure the filename does not end in \r or \n */
+  q = strchr(p, '\r');
+  if(q)
+    *q = '\0';
+
+  q = strchr(p, '\n');
+  if(q)
+    *q = '\0';
+
+  if(copy != p)
+    memmove(copy, p, strlen(p) + 1);
+
+#if defined(_WIN32) || defined(MSDOS)
+  {
+    char *sanitized;
+    SANITIZEcode sc = sanitize_file_name(&sanitized, copy, 0);
+    tool_safefree(copy);
+    if(sc)
+      return NULL;
+    copy = sanitized;
+  }
+#endif /* _WIN32 || MSDOS */
+
+  return copy;
+}
+
+int tool_write_headers(struct HdrCbData *hdrcbdata, FILE *stream)
+{
+  struct curl_slist *h = hdrcbdata->headlist;
+  int rc = 1;
+  while(h) {
+    /* not "handled", just show it */
+    size_t len = strlen(h->data);
+    if(len != fwrite(h->data, 1, len, stream))
+      goto fail;
+    h = h->next;
+  }
+  rc = 0; /* success */
+fail:
+  curl_slist_free_all(hdrcbdata->headlist);
+  hdrcbdata->headlist = NULL;
+  return rc;
+}
+
+/*
+ * Write etag to file when --etag-save option is given.
+ */
+static size_t save_etag(const char *etag_h, const char *endp,
+                        struct OutStruct *etag_save)
+{
+  const char *eot = endp - 1;
+  if(*eot == '\n') {
+    while(ISBLANK(*etag_h) && (etag_h < eot))
+      etag_h++;
+    while(ISSPACE(*eot))
+      eot--;
+
+    if(eot >= etag_h) {
+      size_t etag_length = eot - etag_h + 1;
+      /*
+       * Truncate the etag save stream, it can have an existing etag value.
+       */
+#ifdef HAVE_FTRUNCATE
+      if(ftruncate(fileno(etag_save->stream), 0)) {
+        return CURL_WRITEFUNC_ERROR;
+      }
+#else
+      if(fseek(etag_save->stream, 0, SEEK_SET)) {
+        return CURL_WRITEFUNC_ERROR;
+      }
+#endif
+
+      fwrite(etag_h, 1, etag_length, etag_save->stream);
+      /* terminate with newline */
+      fputc('\n', etag_save->stream);
+      (void)fflush(etag_save->stream);
+    }
+  }
+  return 0; /* ok */
+}
+
+/*
+ * This function sets the filename where output shall be written when curl
+ * options --remote-name (-O) and --remote-header-name (-J) have been
+ * simultaneously given and additionally server returns an HTTP
+ * Content-Disposition header specifying a filename property.
+ */
+static size_t content_disposition(const char *str, const char *end,
+                                  size_t cb, struct per_transfer *per)
+{
+  struct HdrCbData *hdrcbdata = &per->hdrcbdata;
+  struct OutStruct *outs = &per->outs;
+
+  if((cb > 20) && checkprefix("Content-disposition:", str)) {
+    const char *p = str + 20;
+    /* look for the 'filename=' parameter (encoded filenames (*=) are not
+       supported) */
+    for(;;) {
+      char *filename;
+      size_t len;
+
+      while((p < end) && *p && !ISALPHA(*p))
+        p++;
+      if(p > end - 9)
+        break;
+
+      if(memcmp(p, "filename=", 9)) {
+        /* no match, find next parameter */
+        while((p < end) && *p && (*p != ';'))
+          p++;
+        if((p < end) && *p)
+          continue;
+        else
+          break;
+      }
+      p += 9;
+
+      len = cb - (size_t)(p - str);
+      filename = parse_filename(p, len);
+      if(filename) {
+        if(outs->stream) {
+          /* indication of problem, get out! */
+          curlx_free(filename);
+          return CURL_WRITEFUNC_ERROR;
+        }
+
+        if(per->config->output_dir) {
+          outs->filename = curl_maprintf("%s/%s", per->config->output_dir,
+                                         filename);
+          curlx_free(filename);
+          if(!outs->filename)
+            return CURL_WRITEFUNC_ERROR;
+        }
+        else
+          outs->filename = filename;
+
+        outs->is_cd_filename = TRUE;
+        outs->regular_file = TRUE;
+        outs->fopened = FALSE;
+        outs->alloc_filename = TRUE;
+        hdrcbdata->honor_cd_filename = FALSE; /* done now! */
+        if(!tool_create_output_file(outs, per->config))
+          return CURL_WRITEFUNC_ERROR;
+        if(tool_write_headers(&per->hdrcbdata, outs->stream))
+          return CURL_WRITEFUNC_ERROR;
+      }
+      break;
+    }
+    if(!outs->stream && !tool_create_output_file(outs, per->config))
+      return CURL_WRITEFUNC_ERROR;
+    if(tool_write_headers(&per->hdrcbdata, outs->stream))
+      return CURL_WRITEFUNC_ERROR;
+  } /* content-disposition handling */
+
+  if(hdrcbdata->honor_cd_filename &&
+     hdrcbdata->config->show_headers) {
+    /* still awaiting the Content-Disposition header, store the header in
+       memory. Since it is not null-terminated, we need an extra dance. */
+    char *clone = curl_maprintf("%.*s", (int)cb, str);
+    if(clone) {
+      struct curl_slist *old = hdrcbdata->headlist;
+      hdrcbdata->headlist = curl_slist_append(old, clone);
+      curlx_free(clone);
+      if(!hdrcbdata->headlist) {
+        curl_slist_free_all(old);
+        return CURL_WRITEFUNC_ERROR;
+      }
+    }
+    else {
+      curl_slist_free_all(hdrcbdata->headlist);
+      hdrcbdata->headlist = NULL;
+      return CURL_WRITEFUNC_ERROR;
+    }
+    return cb; /* done for now */
+  }
+
+  return 0; /* ok */
+}
+
+/*
+** callback for CURLOPT_HEADERFUNCTION
+*
+* 'size' is always 1
+*/
+size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
+{
+  struct per_transfer *per = userdata;
+  struct HdrCbData *hdrcbdata = &per->hdrcbdata;
+  struct OutStruct *outs = &per->outs;
+  struct OutStruct *heads = &per->heads;
+  struct OutStruct *etag_save = &per->etag_save;
+  const char *str = ptr;
+  const size_t cb = size * nmemb;
+  const char *end = (char *)ptr + cb;
+  const char *scheme = NULL;
+
+  if(!per->config)
+    return CURL_WRITEFUNC_ERROR;
+
+#ifdef DEBUGBUILD
+  if(size * nmemb > (size_t)CURL_MAX_HTTP_HEADER) {
+    warnf("Header data exceeds write limit");
+    return CURL_WRITEFUNC_ERROR;
+  }
+#endif
+
+#ifdef _WIN32
+  /* Discard incomplete UTF-8 sequence buffered from body */
+  if(outs->utf8seq[0])
+    memset(outs->utf8seq, 0, sizeof(outs->utf8seq));
+#endif
+
+  /*
+   * Write header data when curl option --dump-header (-D) is given.
+   */
+
+  if(per->config->headerfile && heads->stream) {
+    size_t rc = fwrite(ptr, size, nmemb, heads->stream);
+    if(rc != nmemb)
+      return rc;
+    /* flush the stream to send off what we got earlier */
+    if(fflush(heads->stream)) {
+      errorf("Failed writing headers to %s", per->config->headerfile);
+      return CURL_WRITEFUNC_ERROR;
+    }
+  }
+
+  curl_easy_getinfo(per->curl, CURLINFO_SCHEME, &scheme);
+  scheme = proto_token(scheme);
+  if((scheme == proto_http || scheme == proto_https)) {
+    long response = 0;
+    curl_easy_getinfo(per->curl, CURLINFO_RESPONSE_CODE, &response);
+
+    if((response / 100 != 2) && (response / 100 != 3))
+      /* only care about etag and content-disposition headers in 2xx and 3xx
+         responses */
+      ;
+    else if(per->config->etag_save_file && etag_save->stream &&
+            /* match only header that start with etag (case insensitive) */
+            checkprefix("etag:", str)) {
+      size_t rc = save_etag(&str[5], end, etag_save);
+      if(rc)
+        return rc;
+    }
+
+    /* Parse the content-disposition header. When honor_cd_filename is true
+       other headers may be stored until the content-disposition header is
+       reached, at which point the saved headers can be written. That means
+       the content_disposition() may return an rc when it has saved a
+       different header for writing later. */
+    else if(hdrcbdata->honor_cd_filename) {
+      size_t rc = content_disposition(str, end, cb, per);
+      if(rc)
+        return rc;
+    }
+  }
+  if(hdrcbdata->config->writeout) {
+    char *value = memchr(ptr, ':', cb);
+    if(value) {
+      if(per->was_last_header_empty)
+        per->num_headers = 0;
+      per->was_last_header_empty = FALSE;
+      per->num_headers++;
+    }
+    else if(ptr[0] == '\r' || ptr[0] == '\n')
+      per->was_last_header_empty = TRUE;
+  }
+  if(hdrcbdata->config->show_headers && !outs->out_null &&
+     (scheme == proto_http || scheme == proto_https ||
+      scheme == proto_rtsp || scheme == proto_file)) {
+    /* bold headers only for selected protocols */
+    char *value = NULL;
+
+    if(!outs->stream && !tool_create_output_file(outs, per->config))
+      return CURL_WRITEFUNC_ERROR;
+
+    if(global->isatty &&
+#ifdef _WIN32
+       tool_term_has_bold &&
+#endif
+       global->styled_output)
+      value = memchr(ptr, ':', cb);
+    if(value) {
+      size_t namelen = value - ptr;
+      curl_mfprintf(outs->stream, BOLD "%.*s" BOLDOFF ":", (int)namelen, ptr);
+#ifndef LINK
+      fwrite(&value[1], cb - namelen - 1, 1, outs->stream);
+#else
+      if(curl_strnequal("Location", ptr, namelen)) {
+        write_linked_location(per->curl, &value[1], cb - namelen - 1,
+                              outs->stream);
+      }
+      else
+        fwrite(&value[1], cb - namelen - 1, 1, outs->stream);
+#endif
+    }
+    else
+      /* not "handled", just show it */
+      fwrite(ptr, cb, 1, outs->stream);
+  }
+  return cb;
+}
